@@ -1,5 +1,5 @@
 #############################################################################
-# (c) by Tels 2004 - 2005. Part of Graph::Easy
+# (c) by Tels 2004 - 2005. An empty filler cell. Part of Graph::Easy
 #
 #############################################################################
 
@@ -13,7 +13,7 @@ use vars qw/$VERSION @ISA/;
 
 @ISA = qw/Graph::Easy::Node/;
 
-$VERSION = '0.01';
+$VERSION = '0.02';
 
 #############################################################################
 
@@ -41,6 +41,11 @@ sub _init
   $self;
   }
 
+sub as_ascii
+  {
+  '';
+  }
+
 1;
 __END__
 
